@@ -42,6 +42,7 @@
 #include "gsl/gsl_vector.h"
 
 #include "gzstream.h"
+#include "gemma.h"
 #include "gemma_io.h"
 #include "fastblas.h"
 #include "lapack.h"
@@ -109,27 +110,28 @@ void LMM::WriteFiles() {
   }
 
   auto common_header = [&] () {
-    if (a_mode != 2) {
+    if (a_mode != M_LMM2) {
       outfile << "beta" << "\t";
       outfile << "se" << "\t";
     }
 
-    if (!is_legacy_mode())
-      outfile << "logl_H1" << "\t";  // we may make this an option
+    if (a_mode != M_LMM3 && a_mode != M_LMM9)
+      outfile << "logl_H1" << "\t";
 
     switch(a_mode) {
-    case 1:
+    case M_LMM1:
       outfile << "l_remle" << "\t"
               << "p_wald" << endl;
       break;
-    case 2:
+    case M_LMM2:
+    case M_LMM9:
       outfile << "l_mle" << "\t"
               << "p_lrt" << endl;
       break;
-    case 3:
+    case M_LMM3:
       outfile << "p_score" << endl;
       break;
-    case 4:
+    case M_LMM4:
       outfile << "l_remle" << "\t"
               << "l_mle" << "\t"
               << "p_wald" << "\t"
@@ -142,27 +144,28 @@ void LMM::WriteFiles() {
   auto sumstats = [&] (SUMSTAT st) {
     outfile << scientific << setprecision(6);
 
-    if (a_mode != 2) {
+    if (a_mode != M_LMM2) {
       outfile << st.beta << "\t";
       outfile << st.se << "\t";
     }
 
-    if (!is_legacy_mode())
+    if (a_mode != M_LMM3 && a_mode != M_LMM9)
       outfile << st.logl_H1 << "\t";
 
     switch(a_mode) {
-    case 1:
+    case M_LMM1:
       outfile << st.lambda_remle << "\t"
               << st.p_wald << endl;
       break;
-    case 2:
+    case M_LMM2:
+    case M_LMM9:
       outfile << st.lambda_mle << "\t"
               << st.p_lrt << endl;
       break;
-    case 3:
+    case M_LMM3:
       outfile << st.p_score << endl;
       break;
-    case 4:
+    case M_LMM4:
       outfile << st.lambda_remle << "\t"
               << st.lambda_mle << "\t"
               << st.p_wald << "\t"
@@ -173,6 +176,7 @@ void LMM::WriteFiles() {
   };
 
 
+
   if (!file_gene.empty()) {
     outfile << "geneID" << "\t";
 
@@ -277,7 +281,7 @@ Iterating through a dataset Hi_eval differs and Uab (last row)
 void CalcPab(const size_t n_cvt, const size_t e_mode, const gsl_vector *Hi_eval,
              const gsl_matrix *Uab, const gsl_vector *unused, gsl_matrix *Pab) {
 
-
+#if !defined NDEBUG
   size_t n_index = (n_cvt + 2 + 1) * (n_cvt + 2) / 2; // result size
   auto ni_test = Uab->size1; // inds
   assert(Uab->size1 == Hi_eval->size);
@@ -287,6 +291,7 @@ void CalcPab(const size_t n_cvt, const size_t e_mode, const gsl_vector *Hi_eval,
   assert(Pab->size2 == n_index);
   assert(Hi_eval->size == ni_test);
   // assert(ab->size == n_index);
+#endif // DEBUG
 
   // compute Hi_eval (inds)  * Uab  (inds x n_index) * ab (n_index) and return in Pab (cvt x n_index).
 
@@ -593,6 +598,7 @@ $7 = 3
 $8 = 6
   */
 
+#if !defined NDEBUG
   auto Uab = p->Uab;
   auto ab = p->ab;
   assert(n_index == (n_cvt + 2 + 1) * (n_cvt + 2) / 2);
@@ -606,6 +612,7 @@ $8 = 6
 
   assert(p->e_mode == 0);
   assert(Hi_eval->size == ni_test);
+#endif // DEBUG
 
   CalcPab(n_cvt, p->e_mode, Hi_eval, p->Uab, p->ab, Pab);
   CalcPPab(n_cvt, p->e_mode, HiHi_eval, p->Uab, p->ab, Pab, PPab);
@@ -1111,7 +1118,7 @@ void LogRL_dev12(double l, void *params, double *dev1, double *dev2) {
   return;
 }
 
-void LMM::CalcRLWald(const double &l, const FUNC_PARAM &params, double &beta,
+void LMM::CalcRLWald(const double l, const FUNC_PARAM &params, double &beta,
                      double &se, double &p_wald) {
   size_t n_cvt = params.n_cvt;
   size_t n_index = (n_cvt + 2 + 1) * (n_cvt + 2) / 2;
@@ -1153,7 +1160,8 @@ void LMM::CalcRLWald(const double &l, const FUNC_PARAM &params, double &beta,
   return;
 }
 
-void LMM::CalcRLScore(const double &l, const FUNC_PARAM &params, double &beta,
+
+void LMM::CalcRLScore(const double l, const FUNC_PARAM &params, double &beta,
                       double &se, double &p_score) {
   size_t n_cvt = params.n_cvt;
   size_t n_index = (n_cvt + 2 + 1) * (n_cvt + 2) / 2;
@@ -1194,7 +1202,6 @@ void LMM::CalcRLScore(const double &l, const FUNC_PARAM &params, double &beta,
   gsl_matrix_free(Pab);
   gsl_vector_safe_free(Hi_eval);
   gsl_vector_safe_free(v_temp);
-  return;
 }
 
 void CalcUab(const gsl_matrix *UtW, const gsl_vector *Uty, gsl_matrix *Uab) {
@@ -1415,7 +1422,7 @@ void LMM::AnalyzeGene(const gsl_matrix *U, const gsl_vector *eval,
     CalcUab(UtW, Uty, Uab);
     FUNC_PARAM param0 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
-    if (a_mode == 2 || a_mode == 3 || a_mode == 4) {
+    if (a_mode == M_LMM2 || a_mode == M_LMM3 || a_mode == M_LMM4) {
       CalcLambda('L', param0, l_min, l_max, n_region, l_H0, logl_H0);
     }
 
@@ -1424,16 +1431,16 @@ void LMM::AnalyzeGene(const gsl_matrix *U, const gsl_vector *eval,
     FUNC_PARAM param1 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
     // 3 is before 1.
-    if (a_mode == 3 || a_mode == 4) {
+    if (a_mode == M_LMM3 || a_mode == M_LMM4) {
       CalcRLScore(l_H0, param1, beta, se, p_score);
     }
 
-    if (a_mode == 1 || a_mode == 4) {
+    if (a_mode == M_LMM1 || a_mode == M_LMM4) {
       CalcLambda('R', param1, l_min, l_max, n_region, lambda_remle, logl_H1);
       CalcRLWald(lambda_remle, param1, beta, se, p_wald);
     }
 
-    if (a_mode == 2 || a_mode == 4) {
+    if (a_mode == M_LMM2 || a_mode == M_LMM4) {
       CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
       p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_H0), 1);
     }
@@ -1523,17 +1530,17 @@ void LMM::Analyze(std::function< SnpNameValues(size_t) >& fetch_snp,
       double logl_H1 = 0.0;
 
       // 3 is before 1.
-      if (a_mode == 3 || a_mode == 4) {
+      if (a_mode == M_LMM3 || a_mode == M_LMM4 || a_mode == M_LMM9 ) {
         CalcRLScore(l_mle_null, param1, beta, se, p_score);
       }
 
-      if (a_mode == 1 || a_mode == 4) {
+      if (a_mode == M_LMM1 || a_mode == M_LMM4) {
         // for univariate a_mode is 1
         CalcLambda('R', param1, l_min, l_max, n_region, lambda_remle, logl_H1);
         CalcRLWald(lambda_remle, param1, beta, se, p_wald);
       }
 
-      if (a_mode == 2 || a_mode == 4) {
+      if (a_mode == M_LMM2 || a_mode == M_LMM9 || a_mode == M_LMM4) {
         CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
         p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_mle_H0), 1);
       }
@@ -1582,7 +1589,7 @@ void LMM::Analyze(std::function< SnpNameValues(size_t) >& fetch_snp,
         continue;
 
       double geno = gs[i];
-      if (std::isnan(geno)) {
+      if (isnan(geno)) {
         gsl_vector_set(x_miss, pos, 1.0);
         n_miss++;
       } else {
@@ -1829,7 +1836,7 @@ void LMM::AnalyzePlink(const gsl_matrix *U, const gsl_vector *eval,
           gsl_matrix_submatrix(UtXlarge, 0, 0, UtXlarge->size1, l);
 
       time_start = clock();
-      eigenlib_dgemm("T", "N", 1.0, U, &Xlarge_sub.matrix, 0.0,
+      fast_dgemm("T", "N", 1.0, U, &Xlarge_sub.matrix, 0.0,
                      &UtXlarge_sub.matrix);
       time_UtX += (clock() - time_start) / (double(CLOCKS_PER_SEC) * 60.0);
 
@@ -1845,17 +1852,17 @@ void LMM::AnalyzePlink(const gsl_matrix *U, const gsl_vector *eval,
         FUNC_PARAM param1 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
         // 3 is before 1, for beta.
-        if (a_mode == 3 || a_mode == 4) {
+        if (a_mode == M_LMM3 || a_mode == M_LMM4) {
           CalcRLScore(l_mle_null, param1, beta, se, p_score);
         }
 
-        if (a_mode == 1 || a_mode == 4) {
+        if (a_mode == M_LMM1 || a_mode == M_LMM4) {
           CalcLambda('R', param1, l_min, l_max, n_region, lambda_remle,
                      logl_H1);
           CalcRLWald(lambda_remle, param1, beta, se, p_wald);
         }
 
-        if (a_mode == 2 || a_mode == 4) {
+        if (a_mode == M_LMM2 || a_mode == M_LMM4) {
           CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
           p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_mle_H0), 1);
         }
