@@ -39,6 +39,7 @@
 #include "gemma_io.h"
 #include "mathfunc.h"
 #include "param.h"
+#include "fastblas.h"
 
 using namespace std;
 
@@ -105,6 +106,11 @@ PARAM::PARAM(void)
       time_total(0.0), time_G(0.0), time_eigen(0.0), time_UtX(0.0),
       time_UtZ(0.0), time_opt(0.0), time_Omega(0.0) {}
 
+PARAM::~PARAM() {
+  if (gsl_r)
+    gsl_rng_free(gsl_r);
+}
+
 // Read files: obtain ns_total, ng_total, ns_test, ni_test.
 void PARAM::ReadFiles(void) {
   string file_str;
@@ -505,6 +511,7 @@ void PARAM::CheckParam(void) {
     error = true;
   }
   if (a_mode != 1 && a_mode != 2 && a_mode != 3 && a_mode != 4 && a_mode != 5 &&
+      a_mode != M_LMM9 &&
       a_mode != 11 && a_mode != 12 && a_mode != 13 && a_mode != 14 &&
       a_mode != 15 && a_mode != 21 && a_mode != 22 && a_mode != 25 &&
       a_mode != 26 && a_mode != 27 && a_mode != 28 && a_mode != 31 &&
@@ -644,7 +651,7 @@ void PARAM::CheckParam(void) {
   // Only LMM option (and one prediction option) can deal with
   // multiple phenotypes and no gene expression files.
   if (n_ph > 1 && a_mode != 1 && a_mode != 2 && a_mode != 3 && a_mode != 4 &&
-      a_mode != 43) {
+      a_mode != 9 && a_mode != 43) {
     cout << "error! the current analysis mode " << a_mode
          << " can not deal with multiple phenotypes." << endl;
     error = true;
@@ -816,6 +823,28 @@ void PARAM::CheckParam(void) {
     flag++;
   }
 
+  // Always set up random environment.
+  gsl_rng_env_setup(); // sets gsl_rng_default_seed
+  const gsl_rng_type *T = gsl_rng_default; // pick up environment GSL_RNG_SEED
+
+  if (randseed >= 0)
+    gsl_rng_default_seed = randseed; // CLI option used
+  else if (gsl_rng_default_seed == 0) { // by default we will randomize the seed
+    time_t rawtime;
+    time(&rawtime);
+    tm *ptm = gmtime(&rawtime);
+
+    gsl_rng_default_seed =
+      (unsigned)(ptm->tm_hour % 24 * 3600 + ptm->tm_min * 60 + ptm->tm_sec);
+  }
+  gsl_r = gsl_rng_alloc(T);
+
+  if (is_debug_mode()) {
+    printf ("GSL random generator type: %s; ", gsl_rng_name (gsl_r));
+    printf ("seed = %lu (option %li); ", gsl_rng_default_seed, randseed);
+    printf ("first value = %lu\n", gsl_rng_get (gsl_r));
+  }
+
   if (flag != 1 && a_mode != 15 && a_mode != 27 && a_mode != 28 &&
       a_mode != 43 && a_mode != 5 && a_mode != 61 && a_mode != 62 &&
       a_mode != 63 && a_mode != 66 && a_mode != 67) {
@@ -892,7 +921,7 @@ void PARAM::CheckParam(void) {
   enforce_fexists(file_anno, "open file");
 
   if (!loco.empty()) {
-    enforce_msg((a_mode >= 1 && a_mode <= 4) || a_mode == 21 || a_mode == 22,
+    enforce_msg((a_mode >= 1 && a_mode <= 4) || a_mode == 9 || a_mode == 21 || a_mode == 22,
                 "LOCO only works with LMM and K");
     // enforce_msg(file_bfile.empty(), "LOCO does not work with PLink (yet)");
     enforce_msg(file_gxe.empty(), "LOCO does not support GXE (yet)");
@@ -920,7 +949,7 @@ void PARAM::CheckParam(void) {
   }
 
   if ((a_mode == 1 || a_mode == 2 || a_mode == 3 || a_mode == 4 ||
-       a_mode == 5 || a_mode == 31) &&
+       a_mode == 5 || a_mode == 9 || a_mode == 31) &&
       (file_kin.empty() && (file_ku.empty() || file_kd.empty()))) {
     cout << "error! missing relatedness file. " << endl;
     error = true;
@@ -1412,7 +1441,7 @@ void compKtoV(const gsl_matrix *G, gsl_matrix *V) {
           gsl_matrix_const_submatrix(G, 0, j * ni_test, ni_test, ni_test);
       gsl_matrix_view KiKj_sub =
           gsl_matrix_submatrix(KiKj, 0, t * ni_test, ni_test, ni_test);
-      eigenlib_dgemm("N", "N", 1.0, &Ki.matrix, &Kj.matrix, 0.0,
+      fast_dgemm("N", "N", 1.0, &Ki.matrix, &Kj.matrix, 0.0,
                      &KiKj_sub.matrix);
       t++;
     }
@@ -1764,6 +1793,7 @@ void PARAM::CalcS(const map<string, double> &mapRS2wA,
     gsl_matrix_view Ksub =
         gsl_matrix_submatrix(K, 0, i * ni_test, ni_test, ni_test);
     CenterMatrix(&Ksub.matrix);
+    // Scale the matrix G such that the mean diagonal = 1.
     ScaleMatrix(&Ksub.matrix);
 
     gsl_matrix_view Asub =
@@ -1959,7 +1989,7 @@ void PARAM::CheckCvt() {
   return;
 }
 
-// Post-process phentoypes and covariates.
+// Post-process phenotypes and covariates.
 void PARAM::ProcessCvtPhen() {
 
   // Convert indicator_pheno to indicator_idv.
@@ -2013,22 +2043,6 @@ void PARAM::ProcessCvtPhen() {
            << "analyzed individuals. " << endl;
     } else {
 
-      // Set up random environment.
-      gsl_rng_env_setup();
-      gsl_rng *gsl_r;
-      const gsl_rng_type *gslType;
-      gslType = gsl_rng_default;
-      if (randseed < 0) {
-        time_t rawtime;
-        time(&rawtime);
-        tm *ptm = gmtime(&rawtime);
-
-        randseed = (unsigned)(ptm->tm_hour % 24 * 3600 + ptm->tm_min * 60 +
-                              ptm->tm_sec);
-      }
-      gsl_r = gsl_rng_alloc(gslType);
-      gsl_rng_set(gsl_r, randseed);
-
       // From ni_test, sub-sample ni_subsample.
       vector<size_t> a, b;
       for (size_t i = 0; i < ni_subsample; i++) {
